
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;P_ISECT
;Poly-InterSECT
;takes a list of points and and a flag
;returns true if the 
; segments intersect on themselves.
;If the flag argument is true 
;then error strings will be printed to the command line
;if an intersection is found.
;
(defun p_isect (lst flag2 / flag lst2 lst3 a b c d n j)

  (setq n 0)                            ;setq
  (repeat (length lst)
    (setq a    (nth n lst)
          lst2 (append lst2 (list a))
          )                             ;setq
    (if (equal 2 (length lst2))
      (setq lst3 (append lst3 (list lst2))
            lst2 (list (cadr lst2))
            )                           ;setq
      )                                 ;if    
    (setq n (+ n 1))                    ;setq
    )                                   ;repeat
  (if (equal 2 (length lst2))
    (setq lst3 (append lst3 (list lst2))) ;setq
    )                                   ;if    
  (setq n 0)                            ;setq
  (while (and (< n (length lst3))
              (not flag)
              )                         ;and
    (setq a (nth n lst3)
          b (cadr a)
          a (car a)
          )                             ;setq
    (setq j (+ n 1))
    (while (and (< j (length lst3))
                (not flag)
                )                       ;and 
      (setq c (nth j lst3)
            d (cadr c)
            c (car c)
            )                           ;setq
      (if (and (not (equal b c 0.000001))
               (not (equal a d 0.000001))
               )                        ;and
        (progn
          (setq flag (inters a b c d))
          (if (and flag
                   flag2
                   )                    ;and
            (progn
              (princ "\nInvalid. Crossing polygon cannot self intersect."
                     ) ;_ end of princ
              (princ flag2)
              )                         ;progn
            )                           ;if
          )                             ;progn
        )                               ;if
      (setq j (+ j 1))                  ;setq
      )                                 ;while
    (setq n (+ n 1))                    ;setq
    )                                   ;while
  flag
  )                                     ;defun p_isect



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;maxminpnt
;takes: a list of points
;returns: a list of 2 points the lower left and the upper right
;
;	maxminpnt	 
(defun maxminpnt (lst / x n a b c d)

  (setq x (car lst)
        a (car x)
        b (cadr x)
        c (car x)
        d (cadr x)
        n 1
        )                               ;setq
  (repeat (max (- (length lst) 1) 0)
    (setq x (nth n lst))                ;setq
    (setq a (min a (car x))
          b (min b (cadr x))
          c (max c (car x))
          d (max d (cadr x))
          )                             ;setq
    (setq n (+ n 1))                    ;setq
    )                                   ;repeat
  (list (list a b)
        (list c d)
        )                               ;list
  )                                     ;defun maxminpnt


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;viewpnts
;returns lower left and upper right coords of current view
(defun viewpnts ( / a b c d x)

(setq b (getvar "viewsize")
      c (car (getvar "screensize"))
      d (cadr (getvar "screensize"))
      a (* b (/ c d))
      x (setq x (getvar "viewctr"))
      x (trans x 1 2)
      c (list (- (car x)  (/ a 2.0))
              (- (cadr x) (/ b 2.0))
              0.0
        );list
      d (list (+ (car x)  (/ a 2.0))
              (+ (cadr x) (/ b 2.0))
              0.0
        );list
      c (trans c 2 1)
      d (trans d 2 1) 
);setq
(list c d)
);defun viewpnts


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;pixel_unit
;returns the size of a single pixel in drawing units.
;value depends on current zoom factor.
;
;pixunit/viewsize = one pixel/yscreensize
;
;pixunit=viewsize/yscreensize 
;
(defun pixel_unit ( / x y x1 y1)
 (setq  y (getvar "viewsize")
       x1 (car (getvar "screensize"))
       y1 (cadr (getvar "screensize"))
        x (* y (/ x1 y1))
 );setq
 (max (abs (/ y y1))
      (abs (/ x x1))
 );max
);defun pixel_unit

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;**PLINE** function takes a list and creates a polyline entity
;the list contains a list of coords.
;and optionaly other lists such as (8 . "LAYER")
;				   (40 . WIDTH)
;				   (62 . COLOR)
;	pline	     n a b flag
(defun pline (lst / n a b flag)

  (if (> (length lst) 1)
    (progn
      (if (setq b (assoc 8 lst))        ;setq
        (setq a (append a (list b)))    ;setq then
        )                               ;if
      (if (setq b (assoc 40 lst))       ;setq
        (setq a (append a
                        (list b)
                        (list (cons 41 (cdr b))
                              )         ;list
                        )               ;append
              )                         ;setq then
        )                               ;if
      (if (setq b (assoc 62 lst))       ;setq
        (setq a (append a (list b)))    ;setq then
        )                               ;if
      )                                 ;progn then
    (setq flag T
          b    (car lst)
          )                             ;setq else only a coord list was provided
    )                                   ;if
  (setq n 0)
  (while (and (not flag)
              (< n (length lst))
              )                         ;and
    (if (not (member (nth n lst) a))
      (setq b    (nth n lst)
            flag T
            )                           ;setq then
      )                                 ;if
    (setq n (+ n 1))                    ;setq
    )                                   ;while
  (entmake (append (list '(0 . "POLYLINE")) a)) ;entmake
  (setq n 0)
  (repeat (length b)
    (entmake (list '(0 . "VERTEX")
                   (append (list 10) (nth n b))
                   )                    ;list
             )                          ;entmake
    (setq n (+ n 1))                    ;setq
    )                                   ;repeat
  (entmake '((0 . "SEQEND")))           ;entmake
  (princ)
  )                                     ;defun pline

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;**VTLIST** takes a list or a polyline ent name
;RETURNS a list of vertecies in
; WORLD if a list is provided '(na code) and code=0
; CURRENT UCS coords are returned if only na is provided as an argument
;or if a lst is provided and code=1
;
;	
(defun vtlist (na / dxf e1 lst lst2 code n flag a z)

                                        ;local function
  (defun dxf (a b /) (cdr (assoc a b))) ;defun
  (if (equal (type na)
             (type (list 1))
             )                          ;equal
    (setq code (cadr na)
          na   (car na)
          )                             ;setq then
    (setq code 1)                       ;setq else
    )                                   ;if
  (setq e1 (entget na))                 ;setq
  (if (equal 1 (logand 1 (dxf 70 e1)))
    (setq flag 1)
    (setq flag nil)
    )                                   ;if
  (if (equal (dxf 0 e1) "POLYLINE")
    (progn
      (setq na (entnext na)
            e1 (entget na)
            )                           ;setq
      (while (/= "SEQEND" (dxf 0 e1))
        (setq lst (append lst
                          (list (trans (dxf 10 e1) na code)) ;list 
                          )             ;append
              na  (entnext na)
              e1  (entget na)
              )                         ;setq
        )                               ;while
      )                                 ;progn then old polyline
    (progn
      (setq lst e1
            z   (dxf 38 e1)
            )                           ;setq
      (if (not z)
        (setq z 0.0)
        ) ;_ end of if
      (setq n 0)                        ;setq
      (repeat (length lst)
        (setq a (nth n lst))
        (if (equal (car a) 10)
          (setq a    (cdr a)
                a    (list (car a) (cadr a) z)
                lst2 (append lst2
                             (list (trans a na code))
                             )          ;append
                )                       ;setq then
          )                             ;if
        (setq n (+ n 1))                ;setq 
        )                               ;repeat
      (setq lst  lst2
            lst2 nil
            )                           ;setq
      )                                 ;progn else lwpolyline
    )                                   ;if 
  (if (and flag lst)
    (setq lst (append lst (list (car lst)))) ;setq
    )                                   ;if
  lst
  )                                     ;defun vtlist

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;ep_list
;returns entity point list
;returns a list of points on the entity
;args
; na -is a polyline 'pl_point_list' is called
; alt -is an arc segmant error tolerance (altitude)
;
(defun ep_list (na alt / dxf a b c d n e1 lst)

                                        ;local function
  (defun dxf (a b /) (cdr (assoc a b))) ;defun
  (setq e1 (entget na))                 ;setq
  (cond
    ((or (equal (dxf 0 e1) "POLYLINE")
         (equal (dxf 0 e1) "LWPOLYLINE")
         )                              ;or
     (setq lst (pl_point_list na alt))  ;setq
     )                                  ;cond #1
    ((equal (dxf 0 e1) "LINE")
     (setq lst (list (trans (dxf 10 e1) na 1)
                     (trans (dxf 11 e1) na 1)
                     )                  ;list
           )                            ;setq
     )                                  ;cond #2
    ((or (equal (dxf 0 e1) "ARC")
         (equal (dxf 0 e1) "CIRCLE")
         )                              ;or
     (progn
       (setq a (dxf 10 e1)              ;the center point
             b (dxf 40 e1)              ;the radius
             n (dxf 50 e1)              ;the start angle
             c (dxf 51 e1)              ;the end angle
             )                          ;setq
       (if (not n)
         (setq n 0
               c (* 2.0 pi)
               )                        ;setq then it's a circle
         (if (> n c)
           (setq c (+ c (* 2.0 pi)))    ;setq then
           )                            ;if else it's an arc
         )                              ;if
       (setq lst (append lst
                         (get_arc_points
                           a
                           (polar a n b)
                           (polar a c b)
                           (- c n)
                           alt
                           )            ;get_arc_points
                         )              ;append
             )                          ;setq
       (setq lst (lsttrans lst na 1))
       )                                ;progn
     )                                  ;cond #3
    ((or (equal "TEXT" (dxf 0 e1))
         (equal "ATTDEF" (dxf 0 e1))
         )                              ;or
     (command "_.ucs" "_ob" na)
     (setq a (textbox e1)
           b (cadr a)
           a (car a)
           c (* 0.2 (dxf 40 e1))        ;extra room factor
           a (list (- (car a) c) (- (cadr a) c) 0.0)
           b (list (+ (car b) c) (+ (cadr b) c) 0.0)
           a (list a
                   (list (car b) (cadr a) 0.0)
                   b
                   (list (car a) (cadr b) 0.0)
                   a
                   )                    ;list
           a (lsttrans a 1 0)
           )                            ;setq
     (command "_.ucs" "_p")
     (setq lst (lsttrans a 0 1))        ;setq
     )                                  ;cond #4
    ((equal "MTEXT" (dxf 0 e1))
     (bns_ucs_2_mtext na)
     (setq lst (bns_mtextbox e1)
           c   (* 0.2 (dxf 40 e1))      ;extra room factor
           a   (car lst)
           b   (caddr lst)
           a   (list (- (car a) c) (- (cadr a) c) 0.0)
           b   (list (+ (car b) c) (+ (cadr b) c) 0.0)
           lst (list a
                     (list (car b) (cadr a) 0.0)
                     b
                     (list (car a) (cadr b) 0.0)
                     a
                     )                  ;list
           lst (lsttrans lst 1 0)
           )                            ;setq
     (command "_.ucs" "_p")
     (setq lst (lsttrans lst 0 1))      ;setq
     )                                  ;cond #5
    ((equal "ELLIPSE" (dxf 0 e1))
     (setq lst (get_ellipse_points na alt))
     )                                  ;cond #6
    ((equal "IMAGE" (dxf 0 e1))
     (setq lst (image_clip_list na))
     )                                  ;cond #7
    )                                   ;cond close
  lst
  )                                     ;defun ep_list

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun pl_point_list
       (na alt / dxf na2 a b c z p1 p2 p3 e1 lst lst2 code n flag)

                                        ;local function
  (defun dxf (a b /) (cdr (assoc a b))) ;defun

  (if (equal (type na)
             (type (list 1))
             )                          ;equal
    (setq code (cadr na)
          na   (car na)
          )                             ;setq then
    (setq code 1)                       ;setq else
    )                                   ;if
  (setq e1 (entget na))                 ;setq
  (if (equal 1 (logand 1 (dxf 70 e1)))
    (setq flag 1)
    (setq flag nil)
    )                                   ;if
  (setq na2 na)
  (if (equal (dxf 0 e1) "POLYLINE")
    (progn
      (setq na (entnext na)
            e1 (entget na)
            p1 (dxf 10 e1)
            b  (dxf 42 e1)
            )                           ;setq
      (if (not (equal 16 (logand 16 (dxf 70 e1))))
        (setq lst (list p1))
        (setq lst nil)
        )                               ;if     
      (setq na (entnext na)
            e1 (entget na)
            )                           ;setq
      (while (/= "SEQEND" (dxf 0 e1))
        (setq p2 (dxf 10 e1))           ;setq
        (if (not (equal 16 (logand 16 (dxf 70 e1))))
          (progn
            (if (and b
                     (not (equal b 0))
                                        ;added below for cloud problems
                     p1
                     (not (equal p1 p2))
                     )                  ;and
              (setq p3  (pl_arc_info p1 p2 b)
                    lst (append
                          lst
                          (get_arc_points (car p3) p1 p2 (caddr p3) alt)
                          )             ;append
                    )                   ;setq then
              (setq lst (append lst (list p2))) ;setq else
              )                         ;if
            )                           ;progn then not a spline control point
          )                             ;if
        (setq b  (dxf 42 e1)
              na (entnext na)
              e1 (entget na)
              p1 p2
              )                         ;setq
        )                               ;while
      (if flag
        (progn
          (setq p2 (car lst))
          (if (and b
                   (not (equal b 0))
                                        ;added below for cloud problems
                   p1
                   (not (equal p1 p2))
                   )                    ;and
            (setq p3  (pl_arc_info p1 p2 b)
                  lst (append lst
                              (get_arc_points (car p3) p1 p2 (caddr p3) alt)
                              )         ;append
                  )                     ;setq then
            (setq lst (append lst (list p2))) ;setq else
            )                           ;if
          (setq flag nil)
          )                             ;progn then it's closed
        )                               ;if
      )                                 ;progn then old polyline
    (progn
      (setq z   (dxf 38 e1)
            lst (member (assoc 10 e1) e1)
            )                           ;setq 
      (if (not z)
        (setq z 0.0)
        ) ;_ end of if
      (setq n 0)                        ;setq
      (repeat (length lst)
        (setq a (nth n lst))
        (if (equal 10 (car a))
          (setq a    (cons 10 (append (cdr a) (list z)))
                lst2 (append lst2 (list a))
                )                       ;setq
          (progn
            (if (equal 42 (car a))
              (setq lst2 (append lst2 (list a))) ;setq then
              )                         ;if
            )                           ;progn
          )                             ;if 
        (setq n (+ n 1))                ;setq
        )                               ;repeat
      (setq b   (car lst2)
            lst (list (cdr b))
            )                           ;setq
      (if flag
        (setq lst2 (append lst2 (list (car lst2)))
              flag nil
              )                         ;setq
        )                               ;if
      (setq n 1)                        ;setq
      (repeat (- (length lst2) 1)
        (setq a (nth n lst2))           ;setq
        (if (and (equal 10 (car a))
                 (equal 42 (car b))
                 (not (equal 0.0 (cdr b)))
                                        ;added below for cloud problems
                 c
                 (not (equal (cdr a) (cdr c)))
                 )                      ;and  
          (progn
            (setq p3  (pl_arc_info (cdr c) (cdr a) (cdr b))
                  lst (append lst
                              (get_arc_points
                                (car p3)
                                (cdr c)
                                (cdr a)
                                (caddr p3)
                                alt
                                ) ;_ end of get_arc_points
                              )         ;append  
                  )                     ;setq
            )                           ;progn then get the arc points
          (progn
            (if (equal 10 (car a))
              (setq lst (append lst (list (cdr a))) ;append
                    )                   ;setq then
              )                         ;if 
            )                           ;progn
          )                             ;if
        (setq c b
              b a
              )                         ;setq
        (setq n (+ n 1))                ;setq 
        )                               ;repeat
      (setq lst2 nil)                   ;setq
      )                                 ;progn else lwpolyline
    )                                   ;if 
  (lsttrans lst na2 code)
  )                                     ;defun pl_point_list

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;get_arc_points
;returns a list of points that lie along an arc described 
;by the following arguments:
;takes: 
;  p1 - start point
;  p2 - end point
;  p3 - center point
; ang - delta angle of the arc
; alt - altitude (max error tolerance)
;
(defun get_arc_points (p1 p2 p3 ang alt / sa ea lst b c d)

  (setq b  (distance p1 p2)             ;the radius
        sa (angle p1 p2)                ;the start angle
        ea (+ sa ang)                   ;the end angle
        )                               ;setq
  (setq d (- ea sa))                    ;setq full delta angle of the arc in radians
  (if (not alt)
    (setq c (/ (* 2.0 pi) 9.0))         ;then use default resolution
    (setq c (delta_ang b alt))          ;setq else use altitute specified. 
    )                                   ;if
  (setq c (* c (/ d (abs d))))          ;setq the delta angle increment of the loop
  (if (< (abs (/ d c)) 4.0)
    (setq c (/ d 4.0)
          )                             ;setq then reset c, the delta angle increment of the loop so 
                                        ;at least 4 segments are used
    )                                   ;if
  (repeat (+ (fix (+ (abs (/ d c))
                     0.000001
                     )                  ;plus
                  )                     ;fix
             1
             )                          ;plus
    (if (not (equal (polar p1 sa b) (last lst)))
      (setq lst (append lst
                        (list (polar p1 sa b))
                        )               ;append
            )                           ;setq then
      )                                 ;if
    (setq sa (+ sa c))                  ;setq
    )                                   ;repeat
  (if (not (equal p3 (last lst)))
    (setq lst (append lst (list p3)))   ;setq
    )                                   ;if
  lst
  )                                     ;defun get_arc_points

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;takes endpoints of an arc and bulge
;returns a list containing:
; Center point
; radius
; Delta angle
; distance (arc length)
; altitude
;
(defun pl_arc_info ( p1 p2 bulge / a b c r ang s)

 (setq b bulge 
       c (distance p1 p2)
       a (abs (/ (* c b) 2.0))
       r (/ (+ (expt (/ c 2.0)
                     2.0
               );expt
               (expt a 2.0)
            );plus
            (* 2.0 a)
         );div
     ang (* 2.0 (atan (/ c 2.0)
                      (- r a)
                );atan
         );mult delta angle
       s (* ang r);the length of the arc
     ang (* ang (/ b (abs b)))
      p3 (polar p1 (angle p1 p2) (/ c 2.0))
      p3 (polar p3 
                (+ (angle p1 p2) (/ pi 2.0)) 
                (* (- r a) 
                   (/ b (abs b))
                );mult
         );polar
 );setq
 (list p3 r ang s a)
);defun pl_arc_info

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;delta_ang
;returns the delta angle of an arc with
;the specified altitude and radius
(defun delta_ang ( r a / c ang)
 (setq c (* 2.0
            (sqrt 
              (abs (- (* 2.0 r a)
                      (expt a 2.0)
                   )
              )
            )
         )
     ang (* 2.0 (atan (/ c 2.0)
                      (- r a)
                );atan
         );mult delta angle
 );setq
 ang
);defun delta_ang

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;takes center point,
;      quad point of major axis
;      quad point of minor axis
;      an angle
; returns a point on the ellipse at the specified angle
;Note: This function assumes that points a, b, and c all have the same z 
;      coord.
;
(defun calc_ellipse ( a b c a1 / d1 d2)
 ;p(u)=c+a*cos(u)+b*sin(u)
 ;p(u)=(Cx+a*cos(u))*i+(Cy+a*sin(u))*j
 (setq d1 (+ (car a)
             (* (* -1.0 (distance a b))
                (cos a1)
             )
          );plus
       d2 (+ (cadr a) 
             (* (* -1.0 (distance a c))
                (sin a1)
             )
          );plus
 );setq
 (list d1 d2 (caddr a))
);defun calc_ellipse

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;takes an entity name of an ellipse and a max allowable error distance.
(defun get_ellipse_points
       (na alt / sa da x a1 a2 ang e1 a b c p1 p2 p3 lst)

  (setq e1  (entget na)
        a   (cdr (assoc 10 e1))
        b   (cdr (assoc 11 e1))
        b   (list (- (car a) (car b))
                  (- (cadr a) (cadr b))
                  (- (caddr a) (caddr b))
                  )                     ;list
        x   (cdr (assoc 210 e1))
        a   (trans a na x)
        b   (trans b na x)
        ang (angle a b)
        a1  (cdr (assoc 41 e1))
        a2  (cdr (assoc 42 e1))
        sa  a1
        )                               ;setq
  (if (>= a1 a2)                        ;do some error checking
    (setq a2 (+ a2 pi pi))
    (progn
      (if (> (- a2 a1) (* 2.0 pi))
        (setq a2 (- a2 (* 2.0 pi)))     ;setq then
        )                               ;if
      )                                 ;progn
    )                                   ;if
  (if alt                               ;get the delta_angle for the loop below
    (setq da (abs (delta_ang (distance a b) alt))) ;setq
    (setq da (/ pi 4.0))
    )                                   ;if
  (setq b (rotate_pnt b a (* -1.0 ang))
        c (polar a
                 (/ pi 2.0)
                 (* (cdr (assoc 40 e1)) (distance a b))
                 )                      ;polar
        )                               ;setq
  (while (<= sa a2)
    (setq p3  (calc_ellipse a b c sa)
          p3  (rotate_pnt p3 a ang)
          p3  (trans p3 x 1)
          lst (append lst (list p3))
          )                             ;setq
    (setq sa (+ sa da))
    )                                   ;while
  (if (not (equal sa a2))
    (setq p3  (calc_ellipse a b c a2)
          p3  (rotate_pnt p3 a ang)
          p3  (trans p3 x 1)
          lst (append lst (list p3))
          )                             ;setq then
    )                                   ;if
  lst
  )                                     ;defun get_ellipse_points 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;**M_ASSOC** Multiple-ASSOC
;TAKES:
;a  - search value for assoc
;lst- a list of sub-lists
;RETURNS: a list of all of the sublists that have 'a' as their first element
;
;	m_assoc b lst2
(defun m_assoc (a lst / b lst2)

  (while (setq b (assoc a lst))         ;setq
    (setq lst  (cdr (member b lst))
          lst2 (append lst2 (list b))
          )                             ;setq
    )                                   ;while
  lst2
  )                                     ;defun m_assoc

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;lsttrans
;is like standard autolisp trans but takes a 
;LIST of points and returns a list of translated 
;points.
;
(defun
 lsttrans
 (lst a b / lst2 c n)

 (setq n 0)                             ;setq
 (repeat (length lst)
   (setq c    (nth n lst)
         c    (trans c a b)
         lst2 (append lst2 (list c))
         )                              ;setq
   (setq n (+ n 1))                     ;setq
   )                                    ;repeat
 lst2
);defun lsttrans


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;sets current ucs to be parallel to the extrusion vector 
;specified by p1
;
;This function is used by extrim and exchprop
;
(defun ucs_2_ent ( p1 / )
 (setq p1 (strcat "*"
                  (rtos (car p1) 2 8) ","
                  (rtos (cadr p1) 2 8) ","
                  (rtos (caddr p1) 2 8)
          );strcat
 );setq
 (command "_.ucs" "_za" "*0.0,0.0,0.0" p1)
);defun ucs_2_ent

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Takes a layer name and returns 
;True if the layer is locked, and nil if unlocked.
;
(defun b_layer_locked ( la / na e1)
 (setq na (tblobjname "layer" la)
       e1 (entget na)
 );setq
 (equal 4 
        (logand 4 (cdr (assoc 70 e1)))
 );equal
);defun b_layer_locked



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;clear_prev_ss - Clears the previous selection set in AutoCAD by
;executing the select command and then performing an undo 1.
; NOTE: This function does not verify that undo is enabled and will
;           fail if undo is disabled.
(defun clear_prev_ss ()

  (if (entnext)
    (progn
      (command "_.select" (entlast) (entnext))
      (while (not (equal (getvar "cmdnames") "")) (command ""))
      (command "_.undo" "1")            ;clear any selection sets
      )                                 ;progn then
    )                                   ;if
  )                                     ;defun clear_prev_ss



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun ss_visible ( ss code / na e1 n)
 (if ss
     (progn
      (setq n 0)
      (repeat (sslength ss)
       (setq na (ssname ss n)
             e1 (entget na)
       );setq
       (if (not (assoc 60 e1))
           (setq e1 (append e1 (list (cons 60 code))));setq then
           (setq e1 (subst (cons 60 code) (assoc 60 e1) e1));setq else
       );if
       (entmod e1) 
       (entupd na)
       (setq n (+ n 1));setq
      );repeat 
     );progn
 );if
);defun ss_visible


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Rotate 'pnt' from a base point of 'p1' and through an angle 
;of 'ang' (in radians)
(defun rotate_pnt ( pnt p1 ang / )
 (polar p1 
        (+ (angle p1 pnt) ang) 
        (distance p1 pnt) 
 );polar
);defun rotate_pnt 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;**PSTRIP** function that strips the path off of file name string
(defun pstrip ( a / b)
 (cond
  ((setq b (strsea "\\" a));setq
   (setq b b);setq
  );cond #1
  ((setq b (strsea "/" a));setq
   (setq b b);setq
  );cond #2
  (T
   (setq b (list 0));setq
  );cond #3
 );cond close
 (setq a (substr a (+ (last b) 1) (strlen a)));setq
);defun pstrip

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;**XSTRIP** function that strips the extention off of any
; file name provided as a string if the filename provided does not
; have an extention then it is returned as is
;
;       xstrip st
(defun xstrip ( fna / st)
 (if (and (setq st (strsea "." fna));setq test if there is an extension
          (<= (- (strlen fna) 3) (last st))
     );and
     (setq fna (substr fna
                       1
                       (- (last st) 1)
               );substr
     );setq
 );if
 fna
);defun xstrip

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun strsea (a b / c n)
 (cond
  ((equal "" a)
   (setq c nil)
  );cond #1
  ((not (equal (type b) (type "1")))
   (progn (print "ARGUMENT NOT A STRING!!!!")
          (print b)
          (setq c nil)
   );progn
  );cond #2
  ( T
    (progn
     (setq n 1);setq
     (while (>=
               (+ (- (strlen b) n) 1)
                (strlen a)
            );test while arg.
     (if (equal
                (substr b n (strlen a))
                a
         );equal
         (setq c (append c (list n))
               n (-
                    (+ n
                       (strlen a)
                    );plus
                    1
                 );minus
         );setq
     );if
     (setq n (+ n 1));setq
     );while
    );progn
  );cond #3
 );cond close
 c
);defun strsea



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun vp_on_screen (na / vp e1 a b c p1 p2 p3 p4)
  (if (setq vp (not (equal (getvar "cvport") 1)))
    (command "_.pspace")
    )                                   ;if
  (setq e1 (entget na)
        a  (cdr (assoc 40 e1))
        b  (cdr (assoc 41 e1))
        p1 (cdr (assoc 10 e1))
        p1 (list (- (car p1) (/ a 2.0))
                 (- (cadr p1) (/ b 2.0))
                 (caddr p1)
                 )                      ;list
        p2 (list (+ (car p1) a)
                 (+ (cadr p1) b)
                 (caddr p1)
                 )                      ;list
        p1 (trans p1 na 1)
        p2 (trans p2 na 2)
        p3 (viewpnts)
        c  (* 0.0095 (distance (car p3) (cadr p3)))
        p4 (cadr p3)
        p3 (car p3)
        p3 (list (+ (car p3) c) (+ (cadr p3) c) (caddr p3))
        p4 (list (- (car p4) c) (- (cadr p4) c) (caddr p4))
        )                               ;setq
  (if vp
    (command "_.mspace")
    )                                   ;if
  (or
    (and (>= (car p1) (car p3))
         (>= (cadr p1) (cadr p3))
         (<= (car p1) (car p4))
         (<= (cadr p1) (cadr p4))
         )                              ;and
    (and (>= (car p2) (car p3))
         (>= (cadr p2) (cadr p3))
         (<= (car p2) (car p4))
         (<= (cadr p2) (cadr p4))
         )                              ;and
    (inters p1
            (list (car p2) (cadr p1) (caddr p1)) ;bottom hor
            p3
            (list (car p3) (cadr p4) (caddr p1)) ;left vert
            ) ;_ end of inters
    (inters p1
            (list (car p2) (cadr p1) (caddr p1)) ;bottom hor
            (list (car p4) (cadr p3) (caddr p3))
            p4                          ;right vert
            ) ;_ end of inters
    (inters (list (car p1) (cadr p2) (caddr p1))
            p2                          ;top hor
            p3
            (list (car p3) (cadr p4) (caddr p3)) ;left vert
            ) ;_ end of inters
    (inters (list (car p1) (cadr p2) (caddr p1))
            p2                          ;top hor
            (list (car p4) (cadr p3) (caddr p3))
            p4                          ;right vert
            ) ;_ end of inters
    (inters p3
            (list (car p4) (cadr p3) (caddr p3)) ;bottom hor
            p1
            (list (car p1) (cadr p2) (caddr p1)) ;left vert
            ) ;_ end of inters
    (inters p3
            (list (car p4) (cadr p3) (caddr p3)) ;bottom hor
            (list (car p2) (cadr p1) (caddr p1))
            p2                          ;right vert
            ) ;_ end of inters
    (inters (list (car p3) (cadr p4) (caddr p3))
            p4                          ;top hor
            p2
            (list (car p1) (cadr p2) (caddr p1)) ;left vert
            ) ;_ end of inters
    (inters (list (car p3) (cadr p4) (caddr p3))
            p4                          ;top hor
            (list (car p2) (cadr p1) (caddr p1))
            p2                          ;right vert
            ) ;_ end of inters
    )                                   ;or
  )                                     ;defun vp_on_screen


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;**SSTRIP** function that strips the spaces off of the begining and 
;end of any string
 (defun sstrip ( a / )
  (while (and (not (equal "" a))
              (equal (substr a (strlen a));substr
                     " "
              );equal
         );and
   (setq a (substr a
                   1
                   (- (strlen a) 1)
           );substr
   );setq
  );while
  (while (and (not (equal "" a))
              (equal (substr a 1 1) " ")
         );and
   (setq a (substr a 2));setq
  );while
  a
 );defun sstrip



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;cross_prod returns the cross product (or normal) of two vectors.
(defun cross_prod (p1 p2)
 (list
  (-
     (* (cadr p1) (caddr p2))
     (* (cadr p2) (caddr p1))
  )
  (* -1
      (-
         (* (car p1) (caddr p2))
         (* (car p2) (caddr p1))
      )
  )
  (-
     (* (car p1) (cadr p2))
     (* (car p2) (cadr p1))
  )
 )
);defun cross_prod

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Sets the ucs such that the origin is at the upper left corner of the 
;mtext object and the x axis is aligned with the mtext rotation.
;
(defun bns_ucs_2_mtext ( na / e1 p1 p2 p3 p4 )
 (setq e1 (entget na)
       p1 (cdr (assoc 10 e1))
       p2 (cdr (assoc 210 e1))
       p3 (cdr (assoc 11 e1))
       p1 (trans p1 na 1)
       p2 (trans p2 na 1 T)
       p3 (trans p3 na 1 T)
       p4 (cross_prod p2 p3)
       p2 (list (+ (car p1) (car p2))
                (+ (cadr p1) (cadr p2))
                (+ (caddr p1) (caddr p2))
          );list
       p3 (list (+ (car p1) (car p3))
                (+ (cadr p1) (cadr p3))
                (+ (caddr p1) (caddr p3))
          );list
       p4 (list (+ (car p1) (car p4))
                (+ (cadr p1) (cadr p4))
                (+ (caddr p1) (caddr p4))
          );list
 );setq
 (command "_.ucs" "_3p" p1 p3 p4)
);defun bns_ucs_2_mtext

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; --------------------- MTEXTBOX FUNCTION ------------------------
;   This function returns a list of four points describing the 
;   bounding box of the mtext (MTXT).
; ----------------------------------------------------------------
(defun bns_mtextbox (MTXT / na WDTH HGHT INS JUST ANG P1 P2 P3 P4)
  (if (and (listp MTXT) (= "MTEXT" (cdr (assoc 0 MTXT))))
    (progn
      (setq WDTH (cdr (assoc 42 MTXT))
            HGHT (cdr (assoc 43 MTXT))
              na (cdr (assoc -1 mtxt)) 
             INS (trans (cdr (assoc 10 MTXT)) 
                        na ;changed by rk from 0
                        1
                 );trans
            JUST (cdr (assoc 71 MTXT))
            ANG  0.0 ;changed by rk from (cdr (assoc 50 MTXT)) because ucs_2_mtext
                     ;always aligns the rotation such that it is 0.
      )
      (cond
        ((= JUST 1)
          (setq P1 (polar INS (- ANG (* Pi 0.5)) HGHT) ; lower-left
                P2 (polar P1 ANG WDTH)                 ; lower-right
                P3 (polar INS ANG WDTH)                ; upper-right
                p4 INS                                 ; upper-left
          )
        )
        ((= JUST 2)
          (setq P3 (polar INS ANG (/ WDTH 2))
                P4 (polar INS (+ ANG Pi) (/ WDTH 2))
                P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                P2 (polar P1 ANG WDTH)
          )
        )
        ((= JUST 3)
          (setq P3 INS
                P4 (polar INS (+ ANG Pi) WDTH)
                P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                P2 (polar P1 ANG WDTH)
          )
        )
        ((= JUST 4)
          (setq P4 (polar INS (+ ANG (* Pi 0.5)) (/ HGHT 2))
                P3 (polar P4 ANG WDTH)
                P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                P2 (polar P1 ANG WDTH)
          )
        )
        ((= JUST 5)
          (setq P4 (polar INS (- ANG Pi) (/ WDTH 2))
                P4 (polar P4 (+ ANG (* Pi 0.5)) (/ HGHT 2))
                P3 (polar P4 ANG WDTH)
                P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                P2 (polar P1 ANG WDTH)
          )
        )
        ((= JUST 6)
          (setq P3 (polar INS (+ ANG (* Pi 0.5)) (/ HGHT 2))
                P4 (polar P3 (+ ANG Pi) WDTH)
                P1 (polar P4 (- ANG (* Pi 0.5)) HGHT)
                P2 (polar P1 ANG WDTH)
          )
        )
        ((= JUST 7)
          (setq P1 INS
                P2 (polar P1 ANG WDTH)
                P3 (polar P2 (+ ANG (* Pi 0.5)) HGHT)
                P4 (polar P1 (+ ANG (* Pi 0.5)) HGHT)
          )
        )
        ((= JUST 8)
          (setq P1 (polar INS (+ ANG Pi) (/ WDTH 2))
                P2 (polar P1 ANG WDTH)
                P3 (polar P2 (+ ANG (* Pi 0.5)) HGHT)
                P4 (polar P1 (+ ANG (* Pi 0.5)) HGHT)
          )
        )
        ((= JUST 9)
          (setq P2 INS
                P1 (polar INS (+ ANG Pi) WDTH)
                P3 (polar P2 (+ ANG (* Pi 0.5)) HGHT)
                P4 (polar P1 (+ ANG (* Pi 0.5)) HGHT)
          )
        )
      );cond close
    );progn then it's metxt
    (prompt "\nEntity Not Mtext!")
  );if
  (list P1 P2 P3 P4)
);defun bns_mtextbox



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Takes an entity name of an image.  
;Returns a list clip_boundary points (current UCS). If the image is not clipped
;or has clipping turned off then the image_bounds or max size will 
;be returned.
;  
(defun image_clip_list ( na / e1 e2 lst a )
 (setq e1 (entget na)
       e2 e1
 );setq 
 (while (setq a (assoc 14 e1))
  (setq  e1 (cdr (member a e1))
          a (cdr a)
        lst (append lst (list a))
  );setq
 );while
 
 (if (and (equal (cdr (assoc 280 e2)) 1)
          (equal 4 (logand 4 (cdr (assoc 70 e2))))
     );and
     (progn
      (if (equal 1 (cdr (assoc 71 e2)))
          (setq lst (list (car lst)
                          (list (car (cadr lst)) (cadr (car lst)))
                          (cadr lst)
                          (list (car (car lst)) (cadr (cadr lst)))
                          (car lst)
                    );list
          );setq then the image has a rectangular clip
      );if
      (setq lst (trans_i_2_ucs lst na 1)) 
     );progn 
     (progn
      (setq lst (image_bounds na))
     );progn else the image is not clipped
 );if
 lst
);defun image_clip_list

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Takes: lst  - a list of 2d points obtained from an image entity.
;       na   - entity name of an image 
;       code - 0, 1 or 2 (same as trans 
;                         world coords, current ucs, or screen respectively)
;Returns a list of translated coords.
;
(defun trans_i_2_ucs ( lst na code / dxf e1 p1 p2 p3 p4 p5 a b pnt n lst2 )
 (defun dxf ( a b ) (cdr (assoc a b)))
 (setq e1 (entget na)
       p1 (dxf 10 e1);insert point
       p2 (dxf 11 e1)
       p3 (dxf 12 e1)
       p4 (dxf 13 e1)
       p5 (list 
                (+ (car p1)   (* (cadr p4) (car p3)))
                (+ (cadr p1)  (* (cadr p4) (cadr p3)))
                (+ (caddr p1) (* (cadr p4) (caddr p3)))
          );list
 );setq
 (setq n 0)
 (repeat (length lst)
 (setq pnt (nth n lst)
         a (list 
                 (* (car pnt) (car p2))
                 (* (car pnt) (cadr p2))
                 (* (car pnt) (caddr p2))
           );list
         a (list (+ (car a) (/ (car p2) 2.0))   ;added 1:48 PM 8/20/97
                 (+ (cadr a) (/ (cadr p2) 2.0))
                 (+ (caddr a) (/ (caddr p2) 2.0))
           );list                                  
         b (list (* -1.0 (cadr pnt) (car p3))
                 (* -1.0 (cadr pnt) (cadr p3))
                 (* -1.0 (cadr pnt) (caddr p3))
           );list
         b (list (- (car b) (/ (car p3) 2.0))   ;added 1:48 PM 8/20/97
                 (- (cadr b) (/ (cadr p3) 2.0))
                 (- (caddr b) (/ (caddr p3) 2.0))
           );list 
       pnt (add_delta_xyz p5 a)
       pnt (add_delta_xyz pnt b)
       pnt (trans pnt 
                  0 ;na 
                  code
           )
      lst2 (append lst2 (list pnt))
 );setq
 (setq n (+ n 1));setq
 );repeat
 lst2
);defun trans_i_2_ucs




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun image_bounds ( na / dxf e1 p1 p2 p3 p4 p5 p6 p7 lst)
 (defun dxf ( a b ) (cdr (assoc a b)))
 (setq e1 (entget na)
       p1 (dxf 10 e1);insert point
       p2 (dxf 11 e1)
       p3 (dxf 12 e1)
       p4 (dxf 13 e1)
       p5 (list 
                (+ (car p1)   (* (car p4) (car p2)))
                (+ (cadr p1)  (* (car p4) (cadr p2)))
                (+ (caddr p1) (* (car p4) (caddr p2)))
          );list
       p6 (list 
                (+ (car p5)   (* (cadr p4) (car p3)))
                (+ (cadr p5)  (* (cadr p4) (cadr p3)))
                (+ (caddr p5) (* (cadr p4) (caddr p3)))
          );list
       p7 (list 
                (+ (car p1)   (* (cadr p4) (car p3)))
                (+ (cadr p1)  (* (cadr p4) (cadr p3)))
                (+ (caddr p1) (* (cadr p4) (caddr p3)))
          );list
 );setq
 ;(setq   x (cross_prod p2 ;(delta_xyz p1 p5) ;
 ;                      p3 ;(delta_xyz p1 p7) ;
 ;          );cross_prod
 ;        x (unitv '(0.0 0.0 0.0) x)
 ;);setq
 (setq lst (lsttrans (list p1 p5 p6 p7 p1) 
                     0 ;x ;@rk was na 6:28 PM 8/12/97
                     1
           );lsttrans
 );setq
 lst
);defun image_bounds



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;add_delta_xyz - Add an offset vector to a point to form a new point.
;Takes a point 'a and an offset vector 'b
;Returns a point that is 'b from a.
;
(defun add_delta_xyz ( a b /  c)
(if (and (equal (length a) 3)
         (equal (length b) 3)
    );and
    (setq c (list (+ (car b) (car a))
                  (+ (cadr b) (cadr a))
                  (+ (caddr b) (caddr a))
            );list
    );setq then
    (setq c (list (+ (car b) (car a))
                  (+ (cadr b) (cadr a))
            );list
    );setq else
 );if
 c
);defun add_delta_xyz


;--------------------------extended trim---------------------------------------------------
(defun md_extrim (/ dxf na p1 redraw_it)
                                        ;local function
  (defun dxf (a b /) (cdr (assoc a b))) ;defun
  (setq na md_rechteck)
  (if na
    (progn
                                        ;(setq e1 (entget na));;setq
      (redraw na 3)
      (setq redraw_it T)
      (setq p1 md_halbe)
;;;;;(getpoint "\nPick the side to trim on:"));setq
      (redraw na 4)
      (setq redraw_it nil)
      (if p1
        (etrim na p1)
        )                               ;if
      )                                 ;progn
    )                                   ;if
                                        ;(restore_old_error)
  (princ)
  )                                     ;defun md_extrim

;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;Entity-TRIM function
;takes: na - entity name
;	 a - a point, the side to trim on
;NOTE: This function does not allow for the possible miss of
;      non-continuous linetypes. 
;
(defun etrim (na   a    /    la   dxf  b    d    e1   lst  lst2 n
              j    k    m    ss   na2  na3  na4  x    y    z    flag
              flag2     flag3
              )

                                        ;local function
  (defun dxf (a b /) (cdr (assoc a b))) ;defun

  (setq e1 (entget na))                 ;setq
  (if (or (setq flag (equal (dxf 0 e1) "POLYLINE"))
          (setq flag (equal (dxf 0 e1) "LWPOLYLINE"))
          (equal (dxf 0 e1) "LINE")
          (equal (dxf 0 e1) "CIRCLE")
          (equal (dxf 0 e1) "ARC")
          )                             ;or
    (progn
      (if (and flag
               (equal 8 (logand 8 (dxf 70 e1)))
               )                        ;and
        (setq flag nil)
        )                               ;if
      (setq a (trans a 1 0))            ;setq
      (command "_.ucs" "_View")
      (setq lst (ep_list na nil)
;;;find extents of selected cutting edge object
            lst (maxminpnt lst)
            x   (- (car (cadr lst)) (car (car lst)))
            y   (- (cadr (cadr lst)) (cadr (car lst)))
            x   (* 0.075 x)
            y   (* 0.075 y)
            z   (list x y)
            x   (list (+ (car (cadr lst)) (car z))
                      (+ (cadr (cadr lst)) (cadr z))
                      )                 ;list
            y   (list (- (car (car lst)) (car z))
                      (- (cadr (car lst)) (cadr z))
                      )                 ;list
            )                           ;setq
      (command "_.zoom" "_w" x y)
      (entupd na)
;;;update the ent. so it's curves display smoothly
      (setq lst (ep_list na
                         (/ (pixel_unit) 2.0)
                         ) ;_ end of ep_list
            )                           ;setq
      (if (or (not flag)
              (not (p_isect lst nil))
              )                         ;or 
        (progn                          ;then the object is valid and not a self intersecting polyline.
          (if (and flag
                   (equal (car lst) (last lst) 0.0001)
                   )                    ;and
            (setq flag3 T)              ;then the polyline could potentialy need a second offset
            )                           ;if
          (if (setq la (b_layer_locked (getvar "clayer")))
            (command "_.layer" "_unl" (getvar "clayer") "")
            )                           ;if
          (command "_.pline")
          (setq b nil)
          (setq n 0)                    ;setq
          (repeat (length lst)
            (setq d (nth n lst))
            (if (not (equal d b 0.0001))
              (progn
                (command d)
                (setq lst2 (append lst2 (list d))) ;setq
                (setq b d)              ;setq 
                )                       ;progn
              )                         ;if 
            (setq n (+ n 1))
            )                           ;repeat
          (command "")
          (setq na2 (entlast)
                ss  (ssadd)
                ss  (ssadd na2 ss)
                lst nil
                )                       ;setq
          (ss_visible ss 1)
          (setq lst2 (get_fence_points na2 a lst2 flag3 flag)) ;setq
          (if la
            (command "_.layer" "_lock" (getvar "clayer") "")
            )                           ;if
          (command "_.ucs" "_p")
                                        ;Move the ents to force a display update of the ents to avoid viewres problems. 
                                        ;(setvar "highlight" 0)
          (if (setq ss (ssget "_f" (last lst2)))
            (command "_.move" ss "" "0,0,0" "0,0,0")
            )                           ;if
          (if flag
            (progn
              (if (setq la (b_layer_locked (dxf 8 e1)))
                (command "_.layer" "_unl" (dxf 8 e1) "")
                )                       ;if 
              (ucs_2_ent (dxf 210 e1))
              (command "_.copy" na "" "0,0,0" "0,0,0")
              (entdel na)
              (setq na3 na
                    na  (entlast)
                    )                   ;setq
              (command "_.pedit" na "_w" "0.0" "_x")
              (command "_.ucs" "_p")
              (if la
                (command "_.layer" "_lock" (dxf 8 e1) "")
                )                       ;if 
              )                         ;progn
            )                           ;if
          (command "_.trim" na "")
          (setq m (- (length lst2) 1))  ;setq
          (setq k 0)                    ;@rk
          (repeat (length lst2)
            (setq lst (nth k lst2))
            (setq a (trans (car lst) 0 1))
            (setq n 1)
            (repeat (- (length lst) 1)  ;repeat each fence list
              (setq b (trans (nth n lst) 0 1))
              (if (equal a b 0.0001)
                (setq flag2 T)
                (setq flag2 nil)
                )                       ;if     
              (setq na4 nil)            ;setq
              (setq j 0)                ;setq
              (while (not flag2)        ;repeat each segment of the fence until no new ents are created. 
                (setq na4 (entlast))    ;setq
                (command "_F" a b "")
                (if (and (equal na4 (entlast))
                         (or (not (equal k m))
                             (> j 0)
                             )          ;or
                         )              ;and
                  (setq flag2 T)
                  )                     ;if
                (setq j (+ j 1))        ;setq 
                )                       ;while
              (setq a b)                ;setq
              (setq n (+ n 1))          ;setq
              )                         ;repeat 
            (setq k (+ k 1))
            )                           ;repeat 
          (command "")
          (if flag
            (progn
              (if (setq la (b_layer_locked (dxf 8 e1)))
                (command "_.layer" "_unl" (dxf 8 e1) "")
                )                       ;if 
              (entdel na)               ;get rid of the copy
              (entdel na3)              ;bring back the original
              (if la
                (command "_.layer" "_lock" (dxf 8 e1) "")
                )                       ;if 
              )                         ;progn
            )                           ;if
          )                             ;progn
        (progn
          (command "_.ucs" "_p")
          (princ "\nSelf intersecting edges are not acceptable.")
          )                             ;progn else invalid self intersecting polyline
        )                               ;if 
      (command "_.zoom" "_p")
      )                                 ;progn then it's a most likely a valid entity.
    )                                   ;if
  )                                     ;defun etrim

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun another_offset
       (pl1 pl2 a1 a2 b na2 lst2 a lst3 lst4 / na ss lst da1 da2)

  (setq da1 (abs (- a2 a1)))            ;setq
  (setq da2 (- (* b (max pl2 pl1))
               (/ (* b (abs (- pl2 pl1)))
                  2.0
                  ) ;_ end of /
               ) ;_ end of -
        )                               ;setq
  (if (> (abs (- da2 da1))
         (* 0.01 (max a1 a2))
         ) ;_ end of >
    (progn
      (pline (list lst2))
      (setq na  (entlast)
            na2 (entlast)
            ss  (ssadd)
            ss  (ssadd na ss)
            )                           ;setq
      (ss_visible ss 1)
      (command "_.offset" b na2 a "")
      (if (and (not (equal na (entlast)))
               (setq lst3 (vtlist (entlast)))
               (setq lst3 (intersect_check lst2 lst3 lst4))
               )                        ;and
        (progn
          (ss_visible (ssadd (entlast) (ssadd)) 1)
          (command "_.area" "_ob" (entlast))
          (setq pl2 (getvar "perimeter")
                a2  (getvar "area")
                )                       ;setq
          (setq lst (list (vtlist (list (entlast) 0)))) ;setq
          (entdel (entlast))            ;then offset was a success so delete the ent after getting it's info
          )                             ;progn then
        (if (not (equal na (entlast)))
          (entdel (entlast))
          )                             ;if else
        )                               ;if
      (entdel na2)
      )                                 ;progn then let's do that second offset 
    )                                   ;if
  lst
  )                                     ;defun another_offset

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun get_fence_points (na2  a    M:lst2 flag plflag    /    a1   a2
                         pl1  pl2  b    c    d    n    lst  lst2 lst3
                         lst4 na
                         )

  (if flag
    (progn
      (setq lst2 (cdr M:lst2))            ;setq
      (repeat (fix (/ (length lst2) 2))
        (setq lst2 (append (cdr lst2) (list (car lst2))) ;append
              )                         ;setq
        )                               ;repeat
      (setq lst2 (append lst2 (list (car lst2)))) ;setq
      (command "_.area" "_ob" na2)
      (setq pl1 (getvar "perimeter")
            a1  (getvar "area")
            )                           ;setq
      )                                 ;progn
    (setq lst2 M:lst2)
    )                                   ;if
  (setq a    (trans a 0 1)
        b    (* (getvar "viewsize") 0.05) ;initial offset distance
        n    3.0                        ;number of offsets
        d    (/ b (- n 1))              ;delta offset
        c    (pixel_unit)
        lst4 (viewpnts)
        )                               ;setq
  (while (> b c)
    (setq na (entlast))
    (command "_.offset" b na2 a "")
    (if (and (not (equal na (entlast)))
             (setq lst3 (vtlist (entlast)))
             (or (not plflag)
                 (setq lst3 (intersect_check lst2 lst3 lst4))
                 )                      ;or
             )                          ;and 
      (progn
        (setq lst3 (lsttrans lst3 1 0))
        (ss_visible (ssadd (entlast) (ssadd)) 1)
        (if flag
          (progn
            (command "_.area" "_ob" (entlast))
            (setq pl2 (getvar "perimeter")
                  a2  (getvar "area")
                  )                     ;setq
            )                           ;progn
          )                             ;if 
        (setq lst (append lst (list lst3))) ;setq
        (entdel (entlast))              ;delete the ent after getting it's vertex info
        (if flag
          (setq lst
                 (append
                   lst
                   (another_offset pl1 pl2 a1 a2 b na2 lst2 a lst3 lst4)
                   )                    ;append
                )                       ;setq
          )                             ;if
        )                               ;progn then offset was a success
      (if (not (equal na (entlast)))
        (entdel (entlast))
        )                               ;if else
      )                                 ;if
    (setq b (- b d))                    ;setq
    )                                   ;while
  (setq na (entlast))
  (command "_.offset" c na2 a "")
  (if (and (not (equal na (entlast)))
           (setq lst3 (vtlist (entlast)))
           (or (not plflag)
               (setq lst3 (intersect_check lst2 lst3 lst4))
               )                        ;or
           )                            ;and 
    (progn
      (setq lst3 (lsttrans lst3 1 0))
      (ss_visible (ssadd (entlast) (ssadd)) 1)
      (if flag
        (progn
          (command "_.area" "_ob" (entlast))
          (setq pl2 (getvar "perimeter")
                a2  (getvar "area")
                )                       ;setq
          )                             ;progn
        )                               ;if 
      (setq lst (append lst (list lst3))) ;setq
      (entdel (entlast))                ;then offset was a success so delete the ent after getting it's info
      (if flag
        (setq
          lst (append
                lst
                (another_offset pl1 pl2 a1 a2 c na2 lst2 a lst3 lst4)
                )                       ;append
          )                             ;setq
        )                               ;if
      )                                 ;progn then
    (if (not (equal na (entlast)))
      (entdel (entlast))
      )                                 ;if else
    )                                   ;if
  (entdel na2)
  lst
  )                                     ;defun get_fence_points

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;returns a list of points on screen if the first two lists do not
;contain segments that intersect each other.
; 
(defun intersect_check (lst  lst2 lst3 /    x    x2   y    y2   lst4
                        flag len  len2 a    aa   b    bb   c    d
                        n    j
                        )

  (setq len  (length lst)
        len2 (length lst2)
        x    (car (car lst3))
        x2   (car (cadr lst3))
        y    (cadr (car lst3))
        y2   (cadr (cadr lst3))
        )                               ;setq
  (setq n 0)                            ;setq
  (while (and (not flag)
              (< (+ n 1) len2)
              )                         ;and
    (setq aa   (nth n lst2)
          bb   (nth (+ n 1) lst2)
          a    (truncate_2_view aa bb x y x2 y2)
          b    (truncate_2_view bb aa x y x2 y2)
          lst4 (append lst4 (list a))
          )                             ;setq
    (if (or (not (equal a aa))
            (not (equal b bb))
            )                           ;or
      (setq lst4 (append lst4 (list b)))
      )                                 ;if
    (setq j 0)                          ;setq 
    (while (and (not flag)
                (< (+ j 1) len)
                )                       ;and
      (setq c    (nth j lst)
            d    (nth (+ j 1) lst)
            flag (inters a b c d)
            )                           ;setq
      (setq j (+ j 1))                  ;setq
      )                                 ;while
    (setq n (+ n 1))                    ;setq
    )                                   ;while
  (if (not (equal b (last lst4)))
    (setq lst4 (append lst4 (list b)))  ;setq
    )                                   ;if
  (if (not flag)
    (setq flag lst4)
    (setq flag nil)
    )                                   ;if
  flag
  )                                     ;defun intersect_check

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun truncate_2_view ( a b x y x2 y2 / int)
 (if (and (< (car a) x)
          (setq int (inters a b (list x y 0.0) (list x y2 0.0)))
     );and
     (setq a int)
     (if (and (> (car a) x2)
              (setq int (inters a b (list x2 y 0.0) (list x2 y2 0.0)))
         );and
         (setq a int)
     );if else
 );if
 (if (and (< (cadr a) y)
          (setq int (inters a b (list x y 0.0) (list x2 y 0.0)))
     );and
     (setq a int)
     (if (and (> (cadr a) y2)
              (setq int (inters a b (list x y2 0.0) (list x2 y2 0.0)))
         );and
         (setq a int)
     );if else
 );if
a
);defun truncate_2_view
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;extendet trim;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun C:loe-neu (/)

  (setq md_erste_ecke (getpoint "Erste Ecke zeigen"))
  (setq md_erste_ecke (list (car md_erste_ecke) (cadr md_erste_ecke)))
  (setq md_dritte_ecke
         (getcorner "\nGegenberliegender Eckpunkt"
                    md_erste_ecke
                    ) ;_ end of getcorner
        ) ;_ end of setq
  (setq md_dritte_ecke
         (list (car md_dritte_ecke) (cadr md_dritte_ecke))
        ) ;_ end of setq
  (setq md_zweite_ecke (list (car md_dritte_ecke) (cadr md_erste_ecke)))
  (setq md_vierte_ecke (list (car md_erste_ecke) (cadr md_dritte_ecke)))
  (setq md_halbe
         (list (/ (+ (car md_dritte_ecke) (car md_erste_ecke)) 2)
               (/ (+ (cadr md_dritte_ecke) (cadr md_erste_ecke)) 2)
               ) ;_ end of list
        ) ;_ end of setq
  (command "_pline"          md_erste_ecke     md_zweite_ecke
           md_dritte_ecke    md_vierte_ecke    "s"
           ) ;_ end of command
  (setq md_rechteck (entlast))
  (md_extrim)
  (setq md_as_rest (ssget "_W" md_erste_ecke md_dritte_ecke))
  (if (/= md_as_rest NIL)
    (command "_.erase" md_as_rest "")
    ) ;_ end of if
  (command "_.erase" md_rechteck "")
  )                                     ;ende defun
;|Visual LISP Format Options
(72 2 40 1 T "end of " 60 9 0 0 0 nil T nil T)
;*** KEINEN Text unterhalb des Kommentars hinzufgen! ***|;

